{
            DCP-1 RS-232 Communications Protocol

    (C) Copyright 2004 John B. Stephensen  All Rights Reserved

  This software in source, object or binary form is licensed only for
  personal non-profit educational use in the Amateur Radio Service and
  the license is not transferrable. The software is provided as-is for
  experimental purposes and the author does not warranty its freedom
  from defects or its suitability for any specific application.
 
  This software resides on a primary station and implements a
  communications protocol between the primary station and 1 to 15
  secondary stations. Data is transmitted and received in frames
  containing 0-64 bytes of user data and 2 bytes of protocol control
  information.

  The frame is transmitted using 8-bit characters with even parity
  and one stop bit. Three special characters are used to acheive
  8-bit transparency:

 	  7F (127) is the abort character.
 	  7E (126) is the flag character.
 	  7D (125) is the escape character.

  The characters 7E and 7F never occur within a frame. If they
  occur in the data to be transmitted, 7F is encoded as 7D 5F and
  7E is encoded as 7D 5E. 7D is encoded as 7D 5D. On reception,
  any occurrance of 7D results in its replacement by the following
  character exclusive OR'ed with 20.

  A normal frame consists of the flag character followed by any
  number encoded data characters and ending with a flag character.
  If a transmission error occurs, an abort character is inserted
  into the data field. The receiver then discards the frame. Two
  or more flag characters may occur between frames.

  The frame consists of a user data field preceded by a one byte
  header field and followed by a one byte parity field. The last
  byte of frame is even parity over each bit of the preceding
  bytes. The user data field may be 0 to 64 bytes in length.

 	      0       1                         N      N+1
 	  +--------+--------------/ /--------------+--------+
 	  | HEADER |        USER DATA FIELD        | PARITY |
 	  +--------+--------------/ /--------------+--------+

  The user data field is generated and decoded by the user of the
  protocol. The header and parity bytes are added by the protocol
  implementation described here on transmission and decoded by it
  on reception.

  The header byte contains three subfields, a 4-bit address field
  (ADDR), a 1-bit poll/acknowledge field (PA), a 1-bit sequence
  number field (SN) and a 2-bit port number (PORT). The address
  identifies the secondary station and the port identifies one
  of four communication endpoints within a station. The PA bit
  is set to one on acknowlegement frames sent to the primary
  station. It is also set to one on polling frames sent to the
  secondary stations. It is set to zero in data frames. The
  sequence bit is used to detect receipt of duplicate frames.
  only one frame is outstanding at any time on each port. Port
  3 is used for control. Ports 0-2 are used for general-purpose
  communication. Address 15 is reserved.

 	    7    6    5    4    3    2    1    0
 	  +----+----+----+----+----+----+----+----+
    |        ADDR       | PA | SN |  PORT   |
 	  +----+----+----+----+----+----+----+----+

  The protocol implementation maintains the variables txs[port] and
  rxs[port] with the last transmitted and received sequence numbers.
  These determine the sequence numbers in transmitted data and
  acknowledgement frames. The variables rxf[port] are used for
  receiver flow control. A value of zero indicates that no new
  data is to be accepted. Positive values indicate the number of
  new data frames to be accepted and this number is decremented for
  each new frame received.

  A communications link between the primary station and a secondary station
  is initialized by sending a frame to port 3 with the PA and SN bits set to
  zero. The secondary station resets all sequence numbers to zero and returns
  a copy of the transmitted frame.

  Data is read from a secondary station port by sending a polling fame.
  This frame has the expected sequence number of the data to be returned.
  When the data frame is received, the received sequence number is
  compared to the next expected sequence number (rxs[port]). If they
  are not equal, the frame is a duplicate and is discarded. If the
  sequence numbers are equal, the frame is delivered to the user, and
  the next expected sequence number is incremented.

  Data is written to a secondary station port by sending a data frame
  containing the current transmit sequence number txs[port]. The
  secondary station respondes with an acknowledgement frame containing
  a sequence number.  The received sequence number is compared
  with the transmitted sequence number (txs[port]). If the two are
  equal, the current data frame is acknowledged and the transmit
  sequence number is incremented. If they are not equal, the data frame
  was not accepted and the primary station tries again after a time delay.
  If no acknowledgement frame is received, the transmission is also retried.
  After the maximum number of reties has been used, the primary station
  gives up and notifies the user of an error.

  Statistics for the communications link may be read from a secondary station
  by sending a frame to port 3 with the PA bit set to zero and the SN bit set
  to one. The secondary station returns a frame with an identical header and
  a 32-byte data field containing:

         Byte 1: Zero
         Byte 2: Current receive buffer pool size.
         Byte 3: Current transmit buffer pool size.
      Bytes 4-7: Number of byte parity errors since last reset.
     Bytes 8-11: Number of frame parity errors since last reset.
    Bytes 10-15: Number of byte overrun errors since last reset.
    Bytes 16-19: Number of frame overrun errors since last reset.
    Bytes 20-23: Number of frame length errors since last reset.
    Bytes 24-27: Number of frames received since last reset.
    Bytes 28-31: Number of frames transmitted since last reset.

  1-byte counters have a value between 0 and 255. 4-byte counters are
  transmitted LSB first and have a value between 0 and 4,294,967,295.

  History:

}
unit DCP1Comm;

interface

type
  statstype =
  record
    rxpool: integer;
    txpool: integer;
    byteparity: integer;
    frameparity: integer;
    byteoverrun: integer;
    frameoverrun: integer;
    lengtherror: integer;
    received: integer;
    transmitted: integer;
  end;

function openlink(): boolean;
function resetlink(addr: integer): boolean;
function txdata(link: integer; tdata: PChar; tsize: integer): boolean;
function rxdata(link: integer; rdata: PChar; rsize: integer): integer;
function getstats(addr: integer; var stats: statstype): boolean;
procedure closelink();
procedure reporterror();

var transmitted,received,txerror,rxerror,dataerror: integer;

implementation

uses
  SysUtils,
  WinTypes,
  WinSvc,
  Windows;
var
  f: THandle; {HANDLE} {I/O Port handle}
  txs,rxs: array[0..2] of char;  {transmitter and receiver state info}

{output hexidecimal}
procedure writehex(i: integer);
begin
  if i < 10
  then write(char(i+integer('0')))
  else write(char(i+integer('A')-10));
end;
procedure displayhex(title: string; buffer: PChar; j,k: integer);
var
  i: integer;
begin
  write(title,'[',j,'-',k,']: ');
  for i := j to k
  do begin
    writehex((integer(buffer[i]) shr 4) and $0F);
    writehex(integer(buffer[i]) and $0F);
    write(' ');
  end;
  writeln;
end; {displayhex}

{Retreive I/O error code and display associated message}
procedure reporterror();
var
  ec: DWORD;
  buf: array[0..127] of char;
begin
  ec := GetLastError();
  write('ERROR ',ec,':');
  FormatMessage(
    FORMAT_MESSAGE_FROM_SYSTEM,
    NIL,
    ec,
    0,
    buf,
    127,
    NIL
  );
  writeln(buf);
end; {reporterror()}

{
  uart_int() opens the COM1 port and configures it

  The Flags field in the DCB is defined as follows:
    0 Binary           binary mode, no EOF check    1
    1 Parity           enable parity checking       1
    2 OutxCtsFlow      CTS output flow control      0
    3 OutxDsrFlow      DSR output flow control      0
                                                   (3)
  4,5 DtrControl       DTR flow control type        01
    6 DsrSensitivity   DSR sensitivity              0
    7 TXContinueOnXoff XOFF continues Tx            0
                                                   (1)
    8 OutX             XON/XOFF out flow control    0
    9 InX              XON/XOFF in flow control     0
   10 ErrorChar        enable error replacement     1
   11 Null             enable null stripping        0
                                                   (4)
12,13 RtsControl       RTS flow control             01
   14 AbortOnError     abort reads/writes on error  1
                                                   (5)
  DtrControl and RtsControl are defined as follows:
    0 Disable
    1 Enable
    2 Handshake
    3 Toggle
}
function openlink(): boolean;
var
  s: DCB;
  t: COMMTIMEOUTS;
begin
  openlink := true;
  {reset counters}
  transmitted := 0;
  received := 0;
  txerror := 0;
  rxerror := 0;
  dataerror := 0;
  {open RS-232 port}
  f := CreateFile(
    'COM1',
    GENERIC_READ or GENERIC_WRITE,
    FILE_SHARE_READ or FILE_SHARE_WRITE,
    NIL,
    OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL,
    0
  );
  if (f = INVALID_HANDLE_VALUE)
  then begin
    writeln('OpenFile failed!');
    reporterror;
    openlink := false
  end else begin
    {configure UART}
    s.DCBlength := sizeof(DCB);  {sizeof(DCB)}
    s.BaudRate := CBR_115200;    {current baud rate}
    s.Flags := $00005413;
    s.XonLim := 0;               {transmit XON threshold}
    s.XoffLim := 0;              {transmit XOFF threshold}
    s.ByteSize := 8;             {number of bits/byte, 4-8}
    s.Parity := EVENPARITY;      {0-4=no,odd,even,mark,space}
    s.StopBits := ONESTOPBIT;    {0,1,2 = 1, 1.5, 2}
    s.XonChar := char(0);        {Tx and Rx XON character}
    s.XoffChar := char(0);       {Tx and Rx XOFF character}
    s.ErrorChar := char(127);    {error replacement character}
    s.EofChar := char(126);      {end of input character}
    s.EvtChar := char(126);      {received event character}
    if not SetCommState(f,s)
    then begin
      writeln('SetCommState failed!');
      if not CloseHandle(f)
      then writeln('CloseHandle failed!');
      openlink := false;
    end;
    {set buffer sizes}
    if not SetupComm(f,256,256)
    then
    begin
      writeln('SetupComm failed!');
      if not CloseHandle(f)
      then writeln('CloseHandle failed!');
      openlink := false;
    end;
    {set timer values - immediate return for read}
    t.ReadIntervalTimeout := 2;
    t.ReadTotalTimeoutMultiplier := 1;
    t.ReadTotalTimeoutConstant := 50;
    t.WriteTotalTimeoutMultiplier := 1;
    t.WriteTotalTimeoutConstant := 50;
    if not SetCommTimeouts(f,t)
    then
    begin
      writeln('SetCommTimeouts failed!');
      if not CloseHandle(f)
      then writeln('CloseHandle failed!');
      openlink := false;
    end;
    {set comm event mask to wait for event character}
    if not SetCommMask(f,EV_RXFLAG)
    then
    begin
      writeln('SetCommMask failed!');
      if not CloseHandle(f)
      then writeln('CloseHandle failed!');
      openlink := false;
    end;
  end;
end; {openlink}

{
  exchangeframes()

  This routine transmits osize bytes in odata until a reply is received.
  The bytes in the reply fame are placed in idata and isize is set if the
  parity of the received frame is OK. Otherwise, this function returns false
  and isize is set to zero.
}
function exchangeframes
  (odata: PChar; osize: DWORD; idata: PChar; isize: DWORD): integer;
var
  reply: boolean;
  written, read, iptr, optr, i: DWORD;
  parity, esc: integer;
  txtries: integer;
  buffer: array[0..135] of char;
begin
  exchangeframes := 0;
  reply := false;
  {initialize retransmission counter and send request}
  txtries := 0;
  repeat
    {place opening flag and control field in frame}
    buffer[0] := char(126);
    i := 1; {buffer index}
    parity := 0; {reset frame parity}
    {place data field in buffer while calculating parity}
    for optr := 0 to osize-2
    do begin
      if (odata[optr] >= char(125)) and (odata[optr] <= char(127))
      then begin  {escape sequence}
        buffer[i] := char(125);
        i := i+1;
        buffer[i] := char(integer(odata[optr]) xor 32);
      end else begin  {normal data}
        buffer[i] := odata[optr];
      end; {if}
      i := i+1;
      parity := parity xor integer(odata[optr]);
    end; {for}
    {add parity}
    if (parity >= 125) and (parity <= 127)
    then begin
      buffer[i] := char(125);
      i := i+1;
      buffer[i] := char(parity xor 32);
    end else begin
      buffer[i] := char(parity);
    end; {if}
    i := i+1;
    {add closing flag}
    buffer[i] := char(126);
    i := i+1;
    {display frame being transmitted}
    {$IFDEF DEBUG}
    displayhex('TX',buffer,0,i-1);
    {$ENDIF}
    {transmit frame}
    if not WriteFile(f, buffer, i, written, NIL)
    then begin
      writeln('WriteFile failed!');
      reporterror;
      exit;
    end; {if WriteFile}
    {check bytes sent}
    if written <> i
    then begin
      writeln(written,' of ',i,' bytes written!');
      txerror := txerror+1;
      exit;
    end; {if}
    transmitted := transmitted+1;
    {initialize receiver state}
    parity := 0;
    iptr := 0;
    esc := 0;
    {read until frame accumulated or timeout}
    repeat
      {receive frame}
      if not ReadFile(f, buffer, 4, read, NIL)
      then begin
        writeln('ReadFile failed!');
        reporterror;
        exit;
      end;  {if ReadFile}
      {display incoming data}
      {$IFDEF DEBUG}
      displayhex('RD',buffer,0,read-1);
      {$ENDIF}
      {process received frame}
      if read > 0
      then begin
        {remove escape sequences while copying data to user buffer}
        i := 0;
        while (i < read)
        do begin
          case integer(buffer[i]) of
          125: esc := 32;     {start of escape sequence}
          126: if (iptr > 0) then reply := parity=0;  {ignore opening flags}
          127: parity := -1;  {parity error}
          else  {regular character}
            if iptr < isize
            then begin  {store data if space}
              idata[iptr] := char(integer(buffer[i]) xor esc);  {save data byte}
              parity := parity xor integer(idata[iptr]); {add to incrementatal parity}
            end else begin {else drop data until end of frame}
              parity := -1;
            end; {if}
            iptr := iptr+1; {increment input buffer pointer}
            esc := 0; {escape used}
          end; {case}
          i := i+1; {increment read buffer pointer}
        end; {while}
      end else begin
        rxerror := rxerror+1;
      end; {if read > 0}
    until reply or (read = 0);
    {display received frame}
    {$IFDEF DEBUG}
    displayhex('RX',idata,0,iptr-1);
    {$ENDIF}
    {wait before retransmission}
    txtries := txtries+1;
    sleep(txtries);
  until reply or (txtries >= 8);
  {update error counters}
  if reply
  then begin
    received := received+1;
    exchangeframes := iptr; {return frame size less parity byte}
  end else begin
    exchangeframes := 0;
  end; {if}
end; {exchangeframes()}

{
  getint - retreives 32-bit integer from 4 byte character array
}
function getint(c: PChar; n: integer): integer;
begin
  getint :=
    integer(c[n]) +
    integer(c[n+1])*256 +
    integer(c[n+2])*65536 +
    integer(c[n+3])*1677216;
end; {getint()}

{
  Reset link to secondary station
}
function resetlink(addr: integer): boolean;
var
  tdata,rdata: array[0..1] of char;
  i,retries: integer;
  ack: boolean;
begin
  resetlink := false;
  {check parameters}
  if (addr > 15)
  then exit;
  {set address and port numbers - reset SN}
  for i := 0 to 2
  do begin
    txs[i] := char(addr*16 + i);
    rxs[i] := char(addr*16 + i);
  end; {for}
  retries := 0;
  ack := false;
  repeat
    {send request to link 3}
    tdata[0] := char(addr*16 + 3);
    i := exchangeframes(tdata, 2, rdata, 2);
    {check that request and response match}
    if i > 0
    then ack := tdata[0] = rdata[0];
    if not ack
    then begin
      retries := retries+1;
      sleep(retries);
    end; {if}
  until ack or (retries >= 2);
  resetlink := ack;
end; {resetlink()}

{
  Get statistics from secondary station
}
function getstats(addr: integer; var stats: statstype): boolean;
var
  tdata: array[0..1] of char;
  rdata: array[0..65] of char;
  i,retries: integer;
  ack: boolean;
begin
  retries := 0;
  ack := false;
  repeat
    {send request to link 3}
    tdata[0] := char(addr*16 + 5);
    i := exchangeframes(tdata, 2, rdata, 66);
    {check that request and response match}
    if i >= 32
    then ack := tdata[0] = rdata[0];
    if ack
    then begin
      stats.rxpool := integer(rdata[2]);
      stats.txpool := integer(rdata[3]);
      stats.byteparity := getint(rdata,4);
      stats.frameparity := getint(rdata,8);
      stats.byteoverrun := getint(rdata,12);
      stats.frameoverrun := getint(rdata,16);
      stats.lengtherror := getint(rdata,20);
      stats.received := getint(rdata,24);
      stats.transmitted := getint(rdata,28);
    end else begin
      retries := retries+1;
      sleep(retries);
    end; {if}
  until ack or (retries >= 2);
  getstats := ack;
end; {getstats()}

{
  Transmit data to secondary station and get acknowledgement
}
function txdata(link: integer; tdata: PChar; tsize: integer): boolean;
var
  rdata: array[0..67] of char;
  size, tries: integer;
  ack: boolean;
begin
  tries := 0;
  ack := false;
  repeat
    tries := tries+1;
    {clear ack bit and send request}
    tdata[0] := char(integer(txs[link]) and $F7);
    size := exchangeframes(tdata, tsize, rdata, 2);
    if size > 0
    then begin
      {check that address and port match and PA=1 and SN = expected SN}
      ack := char(integer(txs[link]) or 8) = rdata[0];
      if ack
      then begin
        txs[link] := char(integer(txs[link]) xor 4);
      end else begin
        sleep(tries*10);  {NAK - wait and retry}
      end; {if}
    end; {if}
  until ack or (tries > 4);
  txdata := ack;
end; {txdata()}

{
  Poll secondary station for data
}
function rxdata(link: integer; rdata: PChar; rsize: integer): integer;
var
  tdata: array[0..1] of char;
  size,tries: integer;
  ack: boolean;
begin
  tries := 0;
  ack := false;
  repeat
    tries := tries+1;
    {set poll bit and send request}
    tdata[0] := char(integer(rxs[link]) or 8);
    size := exchangeframes(tdata, 2, rdata, rsize);
    if size > 0
    then begin
      {check that address and port match and PA=0 and SN = expected SN}
      ack := char(integer(rxs[link]) and $F7) = rdata[0];
      if ack
      then begin
        rxs[link] := char(integer(rxs[link]) xor 4);
      end else begin
        sleep(tries*10);  {NAK - wait and retry}
      end; {if}
    end; {if}
  until ack or (tries > 4);
  rxdata := size;
end; {rxdata()}

procedure CloseLink();
begin
  CloseHandle(f);
end;

end.
