{
                DCP-1 SVF FIle Download Program

    (C) Copyright 2004 John B. Stephensen  All Rights Reserved

  This software in source, object or binary form is licensed only for
  personal non-profit educational use in the Amateur Radio Service and
  the license is not transferrable. The software is provided as-is for
  experimental purposes and the author does not warranty its freedom
  from defects or its suitability for any specific application.

  This program reads a file and transmits the contents to the DCP-1
  card to program the FPGA. The program uses port 0. The file must be
  in SVF format. Note that the FREQUENCY, PIO and PIOMAP instructions
  are not supported.

  STATE PDU:
    1    2    3
  +----+----+----+--/
  | 00 | S1 | S2 | ...
  +----+----+----+--/

  RUNTEST PDU:
     1    2    3    4    5    6    7   8
  +----+----+----+----+----+----+----+----+
  | 01 |       RUNCNT      | ES | RS | RC |
  +----+----+----+----+----+----+----+----+

  SIR & SDR PDUs:
    1    2    3    4    5           N
  +----+----+----+----+------/ /------+
  | 0X | LENGTH  | ES |     DATA      |
  +----+----+----+----+------/ /------+

  TYPE Field:
    00 = STATE
    01 = RUNTEST
    02 = SIRI - send TDI
    03 = SIRIO - send TDI and request TDO
    04 = SDRI - send TDI
    05 = SDRIO - send TDI and request TDO

  S1, S2... = sequential TAP states (any value)
  RUNCNT = number of clocks in run state
  RC (run clock) = 0 for JTAG TCK or 1 for SCK
  RS (run state) = RESET, IDLE, IRPAUSE or DRPAUSE
  ES (end state) = RESET, IDLE, IRPAUSE or DRPAUSE
  LENGTH = number of bits in TDI and TDO fields
  DATA = 0-60 bytes of data

  TAP States are encoded as:
    00 = RESET (may be synchronous reset)
    01 = IDLE
    02 = DRSELECT
    03 = DRCAPTURE
    04 = DRSHIFT
    05 = DREXIT1
    06 = DRPAUSE
    07 = DREXIT2
    08 = DRUPDATE
    09 = IRSELECT
    0A = IRCAPTURE
    0B = IRSHIFT
    0C = IREXIT1
    0D = IRPAUSE
    0E = IREXIT2
    0F = IRUPDATE

  History:

}
{$I-}
program DCP1SVF;
{$APPTYPE CONSOLE}

uses
  System,
  SysUtils,
  StrUtils,
  WinTypes,
  WinSvc,
  Windows,
  DCP1Comm in 'DCP1Comm.pas';

type
  statstype =
  record
    rxpool: integer;
    txpool: integer;
    byteparity: integer;
    frameparity: integer;
    byteoverrun: integer;
    frameoverrun: integer;
    lengtherror: integer;
    received: integer;
    transmitted: integer;
  end;

{output hexidecimal}
procedure writehex(i: integer);
begin
  if i < 10
  then write(char(i+integer('0')))
  else write(char(i+integer('A')-10));
end;
procedure displayhex(title: string; buffer: PChar; j,k: integer);
var
  i: integer;
begin
  write(title,'[',j,'-',k,']: ');
  for i := j to k
  do begin
    writehex((integer(buffer[i]) shr 4) and $0F);
    writehex(integer(buffer[i]) and $0F);
    write(' ');
  end;
  writeln;
end; {displayhex}
procedure displayrevhex(title: string; buffer: array of char; j,k: integer);
var
  i: integer;
begin
  write(title,': ');
  for i := k downto j
  do begin
    writehex((integer(buffer[i]) shr 4) and $0F);
    writehex(integer(buffer[i]) and $0F);
  end;
  writeln;
end; {displayhex}

{***test functions to replace comms unit***}
{var
  loopsize: integer;
  loopbuffer: array[0..67] of char;
function openlink(): boolean;
begin
  writeln('OpenLink()');
  openlink := true;
end;
function resetlink(i: integer): boolean;
begin
  writeln('ResetLink(',i,')');
  resetlink := true;
end;
procedure closelink();
begin
  writeln('CloseLink()');
end;
function txdata(link: integer; buffer: PChar; size: integer): boolean;
var
  i: integer;
begin
  displayhex('TxBuffer ',buffer,1,size-2);
  for i := 0 to size
  do loopbuffer[i] := buffer[i];
  loopsize := size;
  txdata := true;
end;
function rxdata(link: integer; buffer: PChar; size: integer): integer;
var
  i: integer;
begin
  for i := 0 to loopsize
  do buffer[i] := loopbuffer[i];
  displayhex('RxBuffer ',buffer,1,loopsize-2);
  rxdata := loopsize;
end;}

{place 32-bit unsigned integer into byte array}
procedure putint(b: PChar; v: integer);
begin
  b[0] := char(v);
  b[1] := char(v div 256);
  b[2] := char(v div 65536);
  b[3] := char(v div 16777216);
end; {putint}
{
  Communications buffers (txbuffer and rxbuffer):
    Use positions 1-64 for user data (0 for header and 65 for parity).
}
var
  txbuffer,rxbuffer: array[0..67] of char;
const
  maskbits: array [0..7] of char =
    (char($FF),char($01),char($03),char($07),char($0F),char($1F),char($3F),char($7F));
{
  Transmit SVF format PDU
}
function txpdu(pdu, size, endstate: integer; tdi: string): boolean;
var
  i,n: integer;
begin
  txpdu := false;
  n := Length(tdi);
  {format PDU}
  txbuffer[1] := char(pdu); {SIR}
  txbuffer[2] := char(size);
  txbuffer[3] := char(size div 256);
  txbuffer[4] := char(endstate);
  {copy TDI string into frame byte array}
  for i := 1 to n
  do txbuffer[i+4] := tdi[i];
  {displayhex('txpdu',txbuffer,0,n+5);}
  {transmit request}
  if not txdata(0, txbuffer, n+6)
  then begin
    writeln('TXPDU: txdata failed!');
    exit;
  end; {if}
  txpdu := true;
end; {txpdu}
{
  Receive SVF format PDU
}
function rxpdu(pdu, bits, endstate: integer; tdo,mask: string): boolean;
var
  e,i,n: integer;
  c: string;
begin
  rxpdu := false;
  {receive response}
  n := rxdata(0,rxbuffer,66);
  {displayhex('rxpdu',rxbuffer,0,n-1);}
  if n < 3
  then begin
    writeln('RXPDU: rxdata failed!');
    exit;
  end else begin
    if rxbuffer[1] <> char(pdu)
    then begin
      if rxbuffer[1] = char(255)
      then displayhex('RXPDU: invalid PDU transmitted',rxbuffer,1,n-2)
      else displayhex('RXPDU: invalid PDU received',rxbuffer,1,n-2);
      exit;
    end; {if}
  end; {if}
  {compare response data if TDO field present}
  if tdo <> ''
  then begin
    n := n - 6; {TDO field size in bytes}
    {XOR data}
    i := 1;
    repeat
      rxbuffer[i+4] := char(integer(rxbuffer[i+4]) xor integer(tdo[i]));
      i := i+1;
    until i > n;
    {remove unused bits}
    rxbuffer[n+4] := char(integer(rxbuffer[n+4]) and integer(maskbits[bits mod 8]));
    {apply mask}
    if mask <> ''
    then begin
      i := 1;
      repeat
        rxbuffer[i+4] := char(integer(rxbuffer[i+4]) and integer(mask[i]));
        i := i+1;
      until i > n;
    end; {if}
    {check result}
    e := 0;
    i := 1;
    repeat
      e := e or integer(rxbuffer[i+4]);
      i := i+1;
    until i > n;
    if e <> 0
    then begin
      displayrevhex('TDO mismatch',rxbuffer,5,n+4);
      write('Type C to continue:'); {continue?}
      readln(c);
      if (c[1] <> 'C') and (c[1] <> 'c')
      then exit;
    end; {if}
  end; {if}
  rxpdu := true;
end; {rxpdu}
{
  Common variables for input file and command line parsering
}
var
  f: TextFile; {file state}
  cp: integer;  {index into and last byte of command buffer}
  command: string;  {buffer for complete command string}
  line: integer; {line number}
{
  read file until complete command accumulated
}
function getcommand(): boolean;
var
  i,j: integer;
  linebuffer: string;
begin
  {discard old contents}
  getcommand := false;
  command := '';
  repeat
    {get next line from file}
    if eof(f) then exit;
    readln(f,linebuffer);
    {display it}
    line := line+1;
    writeln(line,': ',linebuffer);
    {skip comment lines}
    i := AnsiPos('!',linebuffer);
    j := AnsiPos('//',linebuffer);
    if (i = 0) and (j = 0)
    then command := command + linebuffer;
    {check for end of command}
    i := AnsiPos(';',command);
  until i > 0;
  {initialize pointer to current character}
  cp := 1;
  getcommand := true;
end; {of getcommand}
{
  get alphabetic string from command line
}
function getword(): string;
var
  n: integer;
begin
  {skip white space}
  while command[cp] <= ' ' do cp := cp+1;
  n := cp; {mark start of string}
  while (command[cp] >= 'A') and (command[cp] <= 'Z') do cp := cp+1;
  if cp > n
  then getword := AnsiMidStr(command,n,cp-n)
  else getword := '';
end; {getword}
{
  get decimal number from command line
}
function getnum(): integer;
var
  n: integer;
begin
  {skip white space}
  while command[cp] <= ' ' do cp := cp+1;
  n := 0; {reset accumulator}
  while (command[cp] >= '0') and (command[cp] <= '9') do
  begin
    n := 10*n + (integer(command[cp]) - integer('0'));
    cp := cp+1;
  end;
  getnum := n;
end; {getnum}
{
  convert character value to nibble
}
function hex(c: char): integer;
begin
  hex := 0;
  if c >= 'a'
  then hex := integer(c) - integer('a') + 10
  else
    if c >= 'A'
    then hex := integer(c) - integer('A') + 10
    else
      if c >= '0'
      then hex := integer(c) - integer('0');
end; {hex}
{
  Read hexidecimal string from command line and pad it
  with leading zeros to match SIR or SDR size in bits.

  b = result size in bytes
  i = index
  s = size of hex string on command line in bytes
  n = position of '(' in command buffer
  cp = current position in command buffer
  v = accumulated byte value
}
procedure gethex(var data: string; bits: integer);
var
  bytes,nibbles,i,j,n,v: integer;
begin
  {calculate size in bytes and allocate space}
  bytes := (bits+7) div 8;
  SetLength(data,bytes);
  {skip white space}
  while command[cp] <= ' ' do cp := cp+1;
  n := cp; {mark start of number}
  {setup}
  i := 1;
  j := 1;
  {check for hexidecimal number}
  if command[cp] = '(' then
  begin
    {find end of number}
    while command[cp] <> ')' do cp := cp+1;
    nibbles := cp-n-1;
    {check for errors}
    if nibbles > 2*bytes
    then begin
      writeln('GETHEX: value longer than specified length');
      nibbles := 2*bytes;
      writeln('Press RETURN to continue.');
      readln;
    end; {if}
    {and accumulate byte values backwards from ')'}
    v := 0;
    while (i <= nibbles)
    do begin
      if odd(i)
      then v := hex(command[cp-i])
      else v := v + 16*hex(command[cp-i]);
      if (i = nibbles) or not odd(i)
      then begin
        data[j] := char(v);
        j := j+1;
      end; {if}
      i := i+1;
    end; {while}
  end; {if}
  {fill rest of buffer with zeros}
  while (j <= bytes)
  do begin
    data[j] := char(0);
    j := j+1;
  end; {while}
  cp := cp+1; {skip over ')'}
end; {gethex}

{
  Common variables for formatting and parsing PDUs

  State information for TAP control:
    runstate - state for RUNTEST (defaults to IDLE)
    endstate - final state for RUNTEST (defaults to IDLE)
    endsir - final state after SIR (defaults to IDLE)
    endsdr - final state after SDR (defaults to IDLE)
  Header and trailer lengths:
    hirsize - size (bits) of header for instruction register
    hdrsize - size (bits) of header for data register
    tirsize - size (bits) of trailer for instruction register
    tdrsize - size (bits) of trailer for data register
  Buffers for TAP data and masks - one each for SIR and SDR:
    xyRTDI - data to place on TDI, stored MSB first
    xyRTDO - data to be received on TDO, stored MSB first
    xyRMASK - mask for data received on TDI, stored MSB first
    xyRSMASK - mask for data to place on TDO, stored MSB first
  x = H for header, S for normal data and T for trailer
  y = I for instruction register or D for data register
}
var
  runstate,endstate,endsir,endsdr: integer;
  hirsize,hdrsize,tirsize,tdrsize: integer;
  hirtdi,hirtdo,hirmask,hirsmask: string;
  hdrtdi,hdrtdo,hdrmask,hdrsmask: string;
  sirtdi,sirtdo,sirmask,sirsmask: string;
  sdrtdi,sdrtdo,sdrmask,sdrsmask: string;
  tirtdi,tirtdo,tirmask,tirsmask: string;
  tdrtdi,tdrtdo,tdrmask,tdrsmask: string;

{initialize SVF file parser and set default SVF parameters}
procedure initialize();
begin
  runstate := 1; {IDLE}
  endstate := 1;
  endsir := 1;
  endsdr := 1;
  hirsize := 0; {no headers}
  hdrsize := 0;
  tirsize := 0; {no trailers}
  tdrsize := 0;
  sirtdi := ''; {reset parameters}
  sirtdi := '';
  sdrtdo := '';
  sdrtdo := '';
  cp := 0; {no text to parse}
  command := '';
end; {setdefaults}
{
  Tables of key words for parser
    statelist - TAP states
    switchlist - TRST states
    datalist - SIR & SDR parameters
    clocklist - RUNTEXT clock source parameter
}
const
  statelist: array [0..15] of string = ('RESET','IDLE',
    'DRSELECT','DRCAPTURE','DRSHIFT','DREXIT1','DRPAUSE','DREXIT2,','DRUPDATE',
    'IRSELECT','IRCAPTURE','IRSHIFT','IREXIT1','IRPAUSE','IREXIT2,','IRUPDATE');
  trstlist: array [0..3] of string = ('ON','OFF','Z','ABSENT');
  datalist: array [0..3] of string = ('TDI','TDO','MASK','SMASK');
  clocklist: array [0..1] of string = ('TCK','SCK');
{
  parse state command
  p = parameter value or -1 if invalid
  n = number of parameters
}
function state():boolean;
var
  p,n: integer;
  nextstate: array[1..16] of integer;
begin
  state := false;
  {get parameters}
  n := 0;
  repeat
    p := AnsiIndexText(getword(),statelist);
    if p >= 0
    then begin
      n := n+1;
      nextstate[n] := p;
    end;
  until (p < 0) or (n >= 16);
  {at least one state definition required}
  if n < 1
  then begin
    writeln('STATE: No state list!');
    exit;
  end; {if}
  {last state must be stable}
  case nextstate[n] of
  0,1,6,13: ; {OK}
  else
    writeln('STATE: Invalid final state!');
    exit;
  end; {case}
  {create PDU}
  txbuffer[1] := char(0); {STATE}
  for p := 1 to n
  do txbuffer[p+1] := char(nextstate[p]);
  {transmit request}
  if not txdata(0, txbuffer, n+3)
  then begin
    writeln('STATE: txdata failed!');
    exit;
  end; {if}
  {done}
  state := true;
end; {state}
{
  parse runtest command
  p = run state or state parameter (-1 if invalid)
}
function runtest():boolean;
var
  p,runcount,runclock: integer;
begin
  runtest := false;
  {get optional run state - RESET, IDLE, DRPAUSE & IRPAUSE OK}
  {this also becomes the end state if no end state is specified}
  p := AnsiIndexText(getword(),statelist);
  case p of
  -1: ; {not present}
  0,1,6,13:
    begin
      runstate := p; {good run state}
      endstate := p; {becomes default end state}
    end;
  else
    writeln('RUNTEST: Invalid run state!');
    exit; {invalid state}
  end; {case}
  {get mandatory run count - must be positive}
  runcount := getnum();
  if runcount <= 0 then exit;
  {get mandatory run clock}
  runclock := AnsiIndexText(getword(),clocklist);
  if runclock < 0 then exit;
  {check for optional end state parameter}
  if AnsiCompareText(getword,'ENDSTATE') = 0
  then begin
    p := AnsiIndexText(getword(),statelist);
    case p of
    0,1,6,13: endstate := p; {good end state}
    else
      writeln('RUNTEST: Invalid final state!');
      exit; {invalid state}
    end; {case}
  end; {if}
  {format PDU}
  txbuffer[1] := char(1); {RUNTEST}
  putint(@txbuffer[2], runcount);
  txbuffer[6] := char(endstate);
  txbuffer[7] := char(runstate);
  txbuffer[8] := char(runclock);
  {transmit request}
  if not txdata(0, txbuffer, 10)
  then begin
    writeln('RUNTEST: txdata failed!');
    exit;
  end; {if}
  {done}
  runtest := true;
end; {runtest}

{parse sir command}
function sir():boolean;
var
  n,sirsize: integer;
begin
  sir := false;
  {get mandatory length parameter}
  sirsize := getnum();
  if sirsize <= 0 then exit;
  if sirsize > 32 then exit;
  {get optional parameters}
  sirtdo := '';
  repeat
    n := AnsiIndexText(getword(),datalist);
    case n of
    0: gethex(sirtdi,sirsize); {TDI}
    1: gethex(sirtdo,sirsize); {TDO}
    2: gethex(sirmask,sirsize); {MASK}
    3: gethex(sirsmask,sirsize); {SMASK}
    end; {case}
  until n < 0;
  {exchange PDUs - data returned and checked if TDO specified}
  if sirtdo <> ''
  then begin
    if txpdu(3,sirsize,endsir,sirtdi)
    then begin
      if not rxpdu(3,sirsize,endsir,sirtdo,sirmask)
      then exit;
    end else exit;
  end else begin
    if txpdu(2,sirsize,endsir,sirtdi)
    then begin
      if not rxpdu(2,sirsize,endsir,'','')
      then exit;
    end else exit;
  end;
  {done}
  sir := true;
end; {sir}

{parse sdr command}
function sdr():boolean;
var
  n,pos,bits,bytes,nextstate,sdrsize: integer;
begin
  sdr := false;
  {get mandatory length parameter}
  sdrsize := getnum();
  if sdrsize <= 0 then exit;
  if sdrsize > 4000000 then exit;
  {get optional parameters}
  sdrtdo := '';
  repeat
    n := AnsiIndexText(getword(),datalist);
    case n of
    0: gethex(sdrtdi,sdrsize); {TDI}
    1: gethex(sdrtdo,sdrsize); {TDO}
    2: gethex(sdrmask,sdrsize); {MASK}
    3: gethex(sdrsmask,sdrsize); {SMASK}
    end; {case}
  until n < 0;
  {exchange PDUs}
  pos := 1; {first byte to transmit is LSB}
  bits := 480; {bits per frame unless last frame}
  bytes := 60; {bytes per frame}
  nextstate := 4; {SHIFT-DR state}
  repeat
    if sdrsize <= 480 {check if last PDU}
    then begin
      nextstate := endsdr;
      bits := sdrsize;
      bytes := (bits+7) div 8; {adjust in case short}
    end; {if}
    {send PDU, get response and check returned data if TDO specified}
    if sdrtdo <> ''
    then begin
      if txpdu(5,bits,nextstate,AnsiMidStr(sdrtdi,pos,bytes))
      then begin
        if not rxpdu(5,bits,nextstate,
          AnsiMidStr(sdrtdo,pos,bytes),AnsiMidStr(sdrmask,pos,bytes))
        then exit;
      end else exit;
    end else begin
      if txpdu(4,bits,nextstate,AnsiMidStr(sdrtdi,pos,bytes))
      then begin
        if not rxpdu(4,bits,nextstate,'','')
        then exit;
      end else exit;
    end;
    {calculate offset and length at 60 bytes max. per PDU}
    pos := pos + bytes;
    sdrsize := sdrsize - bits;
  until sdrsize <= 0;
  {done}
  sdr := true;
end; {sdr}

{parse hir command}
function hir():boolean;
var
  n: integer;
begin
  hir := false;
  {get mandatory length parameter}
  hirsize := getnum();
  if hirsize < 0 then exit;
  if hirsize > 0 then exit;
  {get optional parameters}
  repeat
    n := AnsiIndexText(getword(),datalist);
    case n of
    0: gethex(hirtdi,hirsize); {TDI}
    1: gethex(hirtdo,hirsize); {TDO}
    2: gethex(hirmask,hirsize); {MASK}
    3: gethex(hirsmask,hirsize); {SMASK}
    end; {case}
  until n < 0;
  {done}
  hir := true;
end; {hir}

{parse hdr command}
function hdr():boolean;
var
  n: integer;
begin
  hdr := false;
  {get mandatory length parameter}
  hdrsize := getnum();
  if hdrsize < 0 then exit;
  if hdrsize > 0 then exit;
  {get optional parameters}
  repeat
    n := AnsiIndexText(getword(),datalist);
    case n of
    0: gethex(hdrtdi,hdrsize); {TDI}
    1: gethex(hdrtdo,hdrsize); {TDO}
    2: gethex(hdrmask,hdrsize); {MASK}
    3: gethex(hdrsmask,hdrsize); {SMASK}
    end; {case}
  until n < 0;
  {done}
  hdr := true;
end; {hdr}

{parse tir command}
function tir():boolean;
var
  n: integer;
begin
  tir := false;
  {get mandatory length parameter}
  tirsize := getnum();
  if tirsize < 0 then exit;
  if tirsize > 0 then exit;
  {get optional parameters}
  repeat
    n := AnsiIndexText(getword(),datalist);
    case n of
    0: gethex(tirtdi,tirsize); {TDI}
    1: gethex(tirtdo,tirsize); {TDO}
    2: gethex(tirmask,tirsize); {MASK}
    3: gethex(tirsmask,tirsize); {SMASK}
    end; {case}
  until n < 0;
  {done}
  tir := true;
end; {tir}

{parse tdr command}
function tdr():boolean;
var
  n: integer;
begin
  tdr := false;
  {get mandatory length parameter}
  tdrsize := getnum();
  if tdrsize < 0 then exit;
  if tdrsize > 0 then exit;
  {get optional parameters}
  repeat
    n := AnsiIndexText(getword(),datalist);
    case n of
    0: gethex(tdrtdi,tdrsize); {TDI}
    1: gethex(tdrtdo,tdrsize); {TDO}
    2: gethex(tdrmask,tdrsize); {MASK}
    3: gethex(tdrsmask,tdrsize); {SMASK}
    end; {case}
  until n < 0;
  {done}
 tdr  := true;
end; {tdr}

{parse endir command}
function endir():boolean;
var
  n: integer;
begin
  endir := false;
  {get parameter - RESET, IDLE, DRPAUSE & IRPAUSE OK}
  n := AnsiIndexText(getword(),statelist);
  case n of
  0,1,6,13: endsir := n;
  else exit; {invalid state}
  end; {case}
  {done}
  endir := true;
end; {endir}

{parse enddr command}
function enddr():boolean;
var
  n: integer;
begin
  enddr := false;
  {get parameter - RESET, IDLE, DRPAUSE & IRPAUSE OK}
  n := AnsiIndexText(getword(),statelist);
  case n of
  0,1,6,13: endsdr := n;
  else exit; {invalid state}
  end; {case}
  {done}
  enddr := true;
end; {enddr}

{parse trst command}
function trst():boolean;
begin
  trst := false;
  {get parameter - must not be ON}
  if AnsiIndexText(getword(),datalist) = 0
  then exit;
  {done}
  trst := true;
end; {trst}

{parse command line}
function parsecommand(): boolean;
const
  commandlist: array[0..10] of string = ('STATE','RUNTEST',
    'SIR','SDR','HIR','HDR','TIR','TDR','ENDIR','ENDDR','TRST');
begin
  parsecommand := false;
  if not getcommand() then exit;
  {get first word and decode command type}
  case AnsiIndexText(getword(),commandlist) of
  0: parsecommand := state();{STATE}
  1: parsecommand := runtest();{RUNTEST}
  2: parsecommand := sir(); {SIR}
  3: parsecommand := sdr(); {SDR}
  4: parsecommand := hir(); {HIR}
  5: parsecommand := hdr(); {HDR}
  6: parsecommand := tir(); {TIR}
  7: parsecommand := tdr();{TDR}
  8: parsecommand := endir(); {ENDIR}
  9: parsecommand := enddr(); {ENDDR}
  10: parsecommand := trst(); {TRST}
  else writeln('Invalid command!');
  end; {case}
end; {of parsecommand}

var
  i,n: integer;
  fn: array [0..79] of char; {file name}
begin
  repeat
    {get file name}
    write('File name: ');
    readln(fn);
    {open file}
    AssignFile(f,'c:\'+fn+'.svf');
    reset(f);
  until ioresult() = 0;
  {initialize SVF parser}
  initialize();
  {initialize UART}
  if not openlink()
  then begin
    writeln('openlink failed (press RETURN to exit)!');
    readln;
    exit;
  end;
  {synchronize port 1}
  if not resetlink(1)
  then begin
    writeln('resetlink failed (press RETURN to exit)!');
    readln;
    exit;
  end;
  {transmit SVF file}
  line := 0;
  while parsecommand() do ;
  {done - close comm port}
  {writeln('Errors=',dataerror,
    ' Transmitted=',transmitted,' Received=',received,
    ' TxError=',txerror,' RxError=',rxerror);}
  closelink();
  writeln('Press RETURN to terminate.');
  readln;
end.


