/*
 *	CIRCULARBUFFERS.H - include file for circularbuffers.as
 *
 *      (c) Copyright 2004 John Stephensen  All Rights Reserved
 *
 * This software in source, object or binary form is licensed only for
 * personal non-profit educational use in the Amateur Radio Service and
 * the license is not transferrable. The software is provided as-is for
 * experimental purposes and the author does not warranty its freedom
 * from defects or its suitability for any specific application.
 *
 * C language function declarations for ARM assembly language
 * circular buffer management library.
 *
 *	cbinit - initialize the buffer - size in 2-bit words
 *	cbchkb - returns number of readable 8-bit bytes in buffer
 *	cbchkh - returns number of readable 16-bit half-words in buffer
 *	cbchkw - returns number of readable 32-bit words in buffer
 *	cbgetb - returns one 8-bit byte from buffer
 *	cbputb - places one 8-bit byte into buffer
 *	cbgeth - returns one 16-bit half-word from buffer
 *	cbputh - places one 16-bit half-word into buffer
 *	cbgetw - returns one 32-bit word from buffer
 *	cbputw - places one 32-bit word into buffer
 *	cbcommit - makes data written since last commit readable
 *	cbrollback - removes data written since last commit
 */
extern void cbinit(char *ram, int size);
extern int cbchkb(char *buffer);
extern int cbchkh(char *buffer);
extern int cbchkw(char *buffer);
extern char cbgetb(char *buffer);
extern short cbgeth(char *buffer);
extern int cbgetw(char *buffer);
extern int cbputb(char *buffer, char data);
extern int cbputh(char *buffer, short data);
extern int cbputw(char *buffer, int data);
extern void cbcommit(char *buffer);
extern void cbrollback(char *buffer);

