/*
 *	JTAG.H - include file for jtag.as
 *
 *      (c) Copyright 2004 John Stephensen  All Rights Reserved
 *
 * This software in source, object or binary form is licensed only for
 * personal non-profit educational use in the Amateur Radio Service and
 * the license is not transferrable. The software is provided as-is for
 * experimental purposes and the author does not warranty its freedom
 * from defects or its suitability for any specific application.
 *
 * ARM assembly language JTAG library:
 *	jinitialize - initialize I/O port and reset FPGA
 *	jtest - clock JTAG port at 1 clock per usec. with TMS value specified
 *	jstate - change TAP state to specified value
 *	jselect - select new TAP state by clocking in 1-32 TMS bits
 *	jscan - scan data bits into and out of DR or IR
 *	jwrite - scan bytes into IR or DR
 * The following registers are used for calling parameters:
 *	R0 = clock transitions (bit string length)
 *	     or new state (jstate)
 *	R1 = right-justified data bits (jselect)
 *	     byte-aligned data address (jwrite and jshift)
 *	R2 = more flag that has the following values:
 *		0 = last block
 *		1 = first or intermediate block
 * jscan replaces each TDI value in the buffer with the scanned TDO value.
 * jscan and jwrite must be called with TAP in SHIFT-DR or SHIFT-IR state.
 * If the flags are zero, the ending state is EXIT1-DR or EXIT1-IR, 
 * otherwise the TAP state is unchanged.
 *
 *  TAP States are encoded as:
 *    00 = RESET	01 = IDLE	02 = DRSELECT	03 = DRCAPTURE
 *    04 = DRSHIFT	05 = DREXIT1	06 = DRPAUSE	07 = DREXIT2
 *    08 = DRUPDATE	09 = IRSELECT	0A = IRCAPTURE	0B = IRSHIFT
 *    0C = IREXIT1	0D = IRPAUSE	0E = IREXIT2	0F = IRUPDATE
 */
extern void jinitialize(void);
extern void jtest(int microseconds, int tms);
extern void jstate(int newstate);
extern void jselect(int clocks, int tms);
extern void jshift(int size, char *tdio, int flags);
extern void jwrite(int size, char *tdi, int flags);

