/*
 *		QUEUES.H - include file for queues.as
 *
 *      (c) Copyright 2004 John Stephensen  All Rights Reserved
 *
 * This software in source, object or binary form is licensed only for
 * personal non-profit educational use in the Amateur Radio Service and
 * the license is not transferrable. The software is provided as-is for
 * experimental purposes and the author does not warranty its freedom
 * from defects or its suitability for any specific application.
 *
 * C language function declarations for ARM assembly language
 * linked list queue management library.
 */

/*
 * Buffer definition - data field size integer number of words
 */
typedef struct
{
	int link;	/* pointer to next list entry */
	int size;	/* size of data field in bytes */
	char data[];	/* data field - array of bytes */
}
buffer;

/*
 * Queue management structure definition
 */
typedef struct
{
	buffer *head;
	buffer *tail;
}
queue;

/*
 * qnew: initialize list to no entries
 * qfirst: add entry to head of list
 * qlast: add entry to tail of list
 * qnext: retreive first entry of list
 * qfill: create buffers and add to list
 */
extern void qnew(queue *list);
extern void qfirst(queue *list, buffer *entry);
extern void qlast(queue *list, buffer *entry);
extern buffer* qnext(queue *list);
extern buffer* qcheck(queue *list);
extern void qfill(queue *list, int *ram, int number, int size);
extern void qmfirst(queue *dst, queue *src);
extern void qmlast(queue *dst, queue *src);
extern int qcount(queue *list);
extern char parity(char *data, int size);
extern void putint(char *data, int value);
extern int getint(char *data);
extern void putzero(char *data);
extern int semaphore(int *address, int increment);


