/*
 *		UART.H - UART driver interface
 *
 * (c) Copyright 2004 John B. Stephensen  All Rights Reserved
 *
 * This software in source, object or binary form is licensed only for
 * personal non-profit educational use in the Amateur Radio Service and
 * the license is not transferrable. The software is provided as-is for
 * experimental purposes and the author does not warranty its freedom
 * from defects or its suitability for any specific application.
 *
 * The UART driver is a system-level program that implements a PPP-like
 * communications protocol for half-duplex 8-bit transparent asynchronous
 * communication between a primary station and 1-15 secondary stations.
 *
 * The UART driver uses the queues package to transfer frames between
 * programs. 
 */
#include    "queues.h"

/*
 * The secondary station address may be set to 0-15.
 * The divisor sets the baud rate as follows:
 *	baud rate = CCLK / divisor / 16
 * The DCP-1 CCLK is 55.296 MHz.
 * The following procedures are available:
 */
extern void uart_init(int address, int divisor);	/* initialize UART */

/*
 * The following queues are available for communication with the driver
 */
extern queue pool;	/* common buffer pool */
extern queue txq[];	/* post buffers for transmission */
extern queue rxq[];	/* get buffers with received frames */
/*
 * The following semaphores are used for receiver flow control
 */
extern int rxf[];	/* number of receive buffers to use */
/*
 * The following functions provide byte-wide I/O
 */
extern void ureset(int link);
extern int uputc(int link, char c);
extern int ugetc(int link, char *c);
extern void uflush(int link);

